using GraphicsManager.Enums;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using Luski.GUI.MainScreen.Interfaces;
using Luski.net;
using Luski.net.Enums;
using Luski.net.Enums.Main;
using Luski.net.JsonTypes;
using Luski.net.Structures.Main;
using OpenTK.Mathematics;
using OpenTK.Windowing.Common;
using OpenTK.Windowing.GraphicsLibraryFramework;

namespace Luski.GUI.MainScreen.UI;

public class Chat : UserControl
{
    public FlowLayout MessageFlow;
    private Label title, desc;
    private Textbox tb;
    private MainSocketTextChannel? Channel;
    public Chat()
    {
        UserControl titlecon;
        Size = new((int)(980 * Globals.Settings.Scale), (int)(866 * Globals.Settings.Scale));
        BackgroundColor = new(50, 50, 50, 255);
        Anchor = ObjectAnchor.All;
        Controls.Add(MessageFlow = new()
        {
            Size = new(Size.X, (int)(761 * Globals.Settings.Scale)),
            Location = new(0, (int)(52 * Globals.Settings.Scale)),
            BackgroundColor = new(40,40,40,255),
            Anchor = ObjectAnchor.All,
            HScrollPixels = Globals.Settings.PerScrollPixels
        });
        Controls.Add(titlecon = new UserControl(){Anchor = ObjectAnchor.Left | ObjectAnchor.Top | ObjectAnchor.Right, Size = new((int)(980 * Globals.Settings.Scale), (int)(52 * Globals.Settings.Scale)), BackgroundColor = BackgroundColor});
        titlecon.Controls.Add(title = new Label(Globals.DefaultFont)
        {
            //Location = new(
               // (int)((26 - (Globals.DefaultFont.PixelHeight / 2)) * Globals.Settings.Scale),
               // (int)((26 * Globals.Settings.Scale) - (Globals.DefaultFont.PixelHeight/2.0)))
        });
        titlecon.Controls.Add(desc = new Label(Globals.DefaultFont){ Color = new(161,161,161,255), Location = new(title.Location.X + title.Size.X + 5, title.Location.Y)});
        Controls.Add(tb = new Textbox(Globals.DefaultFont, Globals.DefaultFont)
        {
            InsideColor = new(28, 28, 28, 255),
            BorderColor = Color4.DarkCyan,
            Location = new((int)(10 * Globals.Settings.Scale), (int)(824 * Globals.Settings.Scale)),
            Size = new((int)(960 * Globals.Settings.Scale), (int)(34 * Globals.Settings.Scale)),
            Anchor = ObjectAnchor.Bottom | ObjectAnchor.Left | ObjectAnchor.Right
        });
        tb.KeyPress += TbOnKeyPress;
        Globals.Luski.MainServer.MessageReceived += LuskiOnMessageReceived;
    }

    private Task TbOnKeyPress(KeyboardKeyEventArgs arg)
    {
        if (arg.Key != Keys.Enter && arg.Key != Keys.KeyPadEnter) return Task.CompletedTask;
        Thread t = new(() => Thr());
        t.Start();
        return Task.CompletedTask;
    }

    private void Thr()
    {
        //Globals.Luski.MainServer.SendMessage(tb.Text, Channel!.Id, CancellationToken.None);
        Window!.Invoke(() => { tb.Text = string.Empty; }); 
    }

    private Task LuskiOnMessageReceived(MainSocketMessage arg)
    {
        if (Channel!.Id != arg.ChannelID) return Task.CompletedTask;
        IRenderObject? reff = null;
        if (MessageFlow.Controls.Length > 0) reff = MessageFlow.Controls[MessageFlow.Controls.Length - 1];
        AddMessage(Window!.TextureManager, arg);
        if (reff is null || (reff.Location.Y + reff.Size.Y <= MessageFlow.Size.Y && reff.Location.X >= 0)) Window.Invoke(() => { MessageFlow.ScrollToBottom();}); 
        
        return Task.CompletedTask;
    }

    public void Clear()
    {
        MessageFlow.Controls.Clear();
        lastm = null;
        lastUser = null;
    }

    public void UpdateTitle(IChannelPick channelPick)
    {
        if (Channel is not null && Channel!.Id == channelPick.Channel.Id) return;
        Channel = channelPick.Channel;
        title.Text = channelPick.Channel.Title;
        //tb.WatermarkFont = Globals.DefaultFont;
        if (channelPick.Channel.Type == ChannelType.DM)
            title.Text = (channelPick.Channel as MainSocketDMChannel)!.User.DisplayName;
        tb.WatermarkText = "Message " + title.Text;
        if (channelPick.Channel.Description is not null)
        {
            desc.Visible = true;
            desc.Text = channelPick.Channel.Description;
            desc.Location = new(title.Location.X + title.Size.X + 5, title.Location.Y);
        }
        else
        {
            desc.Visible = false;
        }
    }

    private MainSocketMessage? lastm;
    private long? lastUser;
    private ChatMessage? LastChatMessage;

    public void AddMessage(TextureManager m, MainSocketMessage Message)
    {
        bool hasbeentenmin = false;
        if (lastm is not null)
            hasbeentenmin =
                new DateTime(2022, 1, 1, 0, 0, 0, 0).AddMilliseconds(lastm.Id >> 22).ToLocalTime().AddMinutes(10) <
                new DateTime(2022, 1, 1, 0, 0, 0, 0).AddMilliseconds(Message.Id >> 22).ToLocalTime();
        lastm = Message;
        if (lastUser is null || lastUser != Message.AuthorID || hasbeentenmin)
        {
            if (Window is null || !Globals.ms.InvokeRequired)
            {
                MessageFlow.Controls.Add(LastChatMessage = new ChatMessage(m, Message));
            }
            else
                Globals.ms.Invoke(() =>
                {
                    MessageFlow.Controls.Add(LastChatMessage = new ChatMessage(m, Message));
                    Window.DrawFrame();
                });
        }
        else
        {
            if (Window is null || !Globals.ms.InvokeRequired)
            {
                LastChatMessage!.AddMessage(Message);
            }
            else
                Globals.ms.Invoke(() => { LastChatMessage!.AddMessage(Message); Window!.DrawFrame(); });
        }

        lastUser = Message.AuthorID;
    }
}