using System.Reflection;
using GraphicsManager;
using GraphicsManager.Enums;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using Luski.net;
using Luski.net.Enums;
using Luski.net.Enums.Main;
using Luski.net.JsonTypes;
using Luski.net.Structures.Main;
using OpenTK.Graphics.OpenGL4;
using OpenTK.Mathematics;

namespace Luski.GUI.MainScreen.UI;

public class FriendRequest : UserControl
{
    private MainScreen Screen;
    public MainSocketRemoteUser User;
    public Rectangle? Accept, Reject;
    
    public FriendRequest(MainScreen Parent, MainSocketRemoteUser User)
    {
        this.User = User;
        this.Screen = Parent;
        Size = new((int)(868 * Globals.Settings.Scale), (int)(62* Globals.Settings.Scale));
        BackgroundColor = new(34, 34, 34, 255);
       // Controls.Add(new Rectangle(Globals.ms.TextureManager.AddTexture(User.GetAvatar(CancellationToken.None).Result, true)) { Location = new((int)(7.5 * Globals.Settings.Scale),(int)(8.5 * Globals.Settings.Scale)), Size = new((int)(29 * Globals.Settings.Scale))});
       Rectangle rr; 
       Controls.Add(rr=new Rectangle(Globals.ms.TextureManager.AddTexture(Tools.GetResourceStream(Assembly.GetExecutingAssembly(),
            "Luski.Resources.Textures.Status.png"))) { Shader = Rectangle.DefaultAlphaShader[Globals.ms.Context],
            Location = new((int)(10 * Globals.Settings.Scale), (int)(10 * Globals.Settings.Scale)),
            Size = new ((int)(42 * Globals.Settings.Scale))
        });
       rr.BackgroundColor = BackgroundColor;
       Texture t = Globals.ms.TextureManager.AddTexture(User.GetAvatar(CancellationToken.None).Result);
       t.Unit = TextureUnit.Texture1;
       rr.Textures.Add(t);
       rr.Shader = Rectangle.DefaultAlphaTextureShader[Globals.ms.Context];
        Controls.Add(new Label(Globals.DefaultFont) { Text = User.DisplayName, Location = new((int)(50 * Globals.Settings.Scale),(int)(14 * Globals.Settings.Scale))});
        Controls.Add(new Label(Globals.DefaultFont) { Text = User.FriendStatus.ToString(), Location = new((int)(50 * Globals.Settings.Scale),(int)(32 * Globals.Settings.Scale))});
        if (User.FriendStatus == FriendStatus.PendingIn)
        {
            Controls.Add(Accept = new Rectangle()
            {
                Anchor = ObjectAnchor.Top | ObjectAnchor.Right, BackgroundColor = Color4.Green, Location = new((int)(166 * Globals.Settings.Scale), (int)(12 * Globals.Settings.Scale)),
                Size = new((int)(28 * Globals.Settings.Scale))
            });
            Controls.Add(Reject = new Rectangle()
            {
                Anchor = ObjectAnchor.Top | ObjectAnchor.Right, BackgroundColor = Color4.Red, Location = new((int)(212 * Globals.Settings.Scale), Accept.Location.Y),
                Size = Accept.Size
            });
            Accept.Clicked += AcceptOnClicked;
            Reject.Clicked += RejectOnClicked;
        }
    }

    private Task RejectOnClicked(IRenderObject arg)
    {
        //_ = Globals.Luski.SendFriendResult(User.Id, false, CancellationToken.None).Result;
        Screen.RemoveFriendRequest(User);
        return Task.CompletedTask;
    }

    private Task AcceptOnClicked(IRenderObject arg)
    {
        //Screen.AddFriend(Globals.Luski.SendFriendResult(User.Id, true, CancellationToken.None).Result);
        Screen.RemoveFriendRequest(User);
        return Task.CompletedTask;
    }
}