using GraphicsManager.Enums;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using Luski.net;
using OpenTK.Mathematics;
using OpenTK.Windowing.Common;
using OpenTK.Windowing.GraphicsLibraryFramework;

namespace Luski.GUI.StartPage.UI;

public class Login : UserControl
{
    private RoundedButton button;
    private Textbox Password, UsernameTextbox;
    private Label ca;
    public event Func<Task>? ChangeToApp;
    public event Func<Task>? ChangeToCa;
    public Login()
    {
        Size = new((int)(312 * Globals.Settings.Scale), (int)(545 * Globals.Settings.Scale));
        Rectangle tt;
        Controls.Add(tt=new Rectangle(Globals.LuskiTexture) { Size = new((int)(179*Globals.Settings.Scale), (int)(189*Globals.Settings.Scale))});
        Label t;
        Controls.Add(t=new Label(Globals.DefaultFont) { Scale = 1.6f, Location = new((int)(199*Globals.Settings.Scale)), Text = "Luski", Color = new(243, 119, 53, 255) });
        t.Location = new((Size.X / 2) - (t.Size.X / 2), t.Location.Y);
        tt.Location = new((Size.X / 2) - (tt.Size.X / 2), tt.Location.Y);
        Controls.Add(UsernameTextbox = new Textbox(Globals.DefaultFont, Globals.DefaultFont) { TextLocation = TextLocation.TrueCenterLeft, WatermarkText = "Username", Location = new((int)(27*Globals.Settings.Scale),(int)(280 * Globals.Settings.Scale)), Size = new((int)(261 * Globals.Settings.Scale),(int)(27 * Globals.Settings.Scale)), InsideColor = new(28,28,28,255), BorderColor = Color4.DarkCyan });
        Controls.Add(Password = new Textbox(Globals.DefaultFont, Globals.DefaultFont) { TextLocation = TextLocation.TrueCenterLeft, WatermarkText = "Password", PasswordChar = '●', Location = new(UsernameTextbox.Location.X,(int)(365 * Globals.Settings.Scale)), Size = UsernameTextbox.Size, InsideColor = new(28, 28, 28, 255), BorderColor = Color4.DarkCyan });
        Controls.Add(ca = new Label(Globals.DefaultFont)   { Location = new(UsernameTextbox.Location.X,(int)(419 * Globals.Settings.Scale)), Text = "Create Account" });
        Controls.Add(button = new(Globals.DefaultFont) {Text = "Login", Location = new(UsernameTextbox.Location.X, (int)(455 * Globals.Settings.Scale)), Size = new(UsernameTextbox.Size.X,(int)(46 * Globals.Settings.Scale)), InsideColor = new(28, 28, 28, 255), BorderColor = Color4.DarkCyan });
        Password.KeyPress += PasswordOnKeyPress;
        UsernameTextbox.KeyPress += UsernameTextboxOnKeyPress;
        button.Clicked += ButtonOnClicked;
        ca.Clicked += CaOnClicked;
        this.WindowLoaded += OnWindowLoaded; 
    }

    private Task OnWindowLoaded(IRenderObject arg)
    {
        UsernameTextbox.Focus();
        return Task.CompletedTask;
    }

    private Task CaOnClicked(IRenderObject arg)
    {
        ChangeToCa!.Invoke();
        return Task.CompletedTask;
    }

    private Task UsernameTextboxOnKeyPress(KeyboardKeyEventArgs arg)
    {
        if (arg.Key != Keys.Tab) return Task.CompletedTask;
        Password.Focus();
        return Task.CompletedTask;
    }

    private Task PasswordOnKeyPress(KeyboardKeyEventArgs arg)
    {
        if (arg.Key != Keys.Enter && arg.Key != Keys.KeyPadEnter && arg.Key != Keys.Tab) return Task.CompletedTask;
        if (arg.Key == Keys.Tab && arg.Shift) UsernameTextbox.Focus();
        if (arg.Key != Keys.Tab ) ButtonOnClicked(Password);
        return Task.CompletedTask;
    }

    private Task ButtonOnClicked(IRenderObject arg)
    {
        try
        {
            Globals.Luski.MainServer.Login(UsernameTextbox.Text, Password.Text, CancellationToken.None);
            ChangeToApp!.Invoke();
        }
        catch (Exception e)
        {
            if (e.Message.Contains("Invaled Login"))
            {
                BlockDraw = true;
                Password.BorderColor = Color4.DarkRed;
                UsernameTextbox.BorderColor = Color4.DarkRed;
                BlockDraw = false;
                TryDraw();
            }
            else Console.WriteLine(e);
        }
        return Task.CompletedTask;
    }
}
