using GraphicsManager;
using GraphicsManager.Enums;
using Luski.GUI.MainScreen.Interfaces;
using Luski.GUI.MainScreen.UI;
using Luski.net.Structures.Main;
using OpenTK.Windowing.Common;
using OpenTK.Windowing.Desktop;

namespace Luski.GUI.Windows;


public class ChatWindow : Window
{
    private static readonly NativeWindowSettings Settings = new()
    {
        Title = "Channel",
        WindowBorder = WindowBorder.Resizable,
        APIVersion = new Version(3, 2),
        StartFocused = true,
        Size = new OpenTK.Mathematics.Vector2i((int)(754 * Globals.Settings.Scale), (int)(667 * Globals.Settings.Scale)),
        Icon = Globals.Icon,
        SharedContext = null
    };

    public ChatWindow(IChannelPick pick) : base(Settings)
    {
        Chat c;
        WindowBorder = WindowBorder.Resizable;
        Controls.Add(c= new Chat()
        {
            Location = new(0),
            Size = new(Size.X, Size.Y),
            Anchor = ObjectAnchor.All
        });
        c.UpdateTitle(pick);
        IReadOnlyList<MainSocketMessage> messages = pick.Channel.GetMessages(CancellationToken.None, Globals.Settings.LoadPerChannel).Result;
                
        foreach (MainSocketMessage message in messages.Reverse())
        {
            c!.AddMessage(TextureManager, message);
        }
        c!.MessageFlow.ScrollToBottom();
    }
}